#include "mainwidget.h"
#include "ui_mainwidget.h"
#include <QDebug>
#include "qcolorlistmodel.h"


#define FILTER_KIND_PROP "filterKind"

MainWidget::MainWidget(QWidget *parent) :
  QWidget(parent),
  ui(new Ui::MainWidget)
{
  ui->setupUi(this);

  //Записываем в именованное свойство радиокнопок тип фильтрации, который они должны включать
  //!!! Именованые свойства могут хранить объекты классов, унасленованных от QObject
  ui->rbHue->setProperty(FILTER_KIND_PROP, QColorFilterModel::Hue);
  ui->rbLightness->setProperty(FILTER_KIND_PROP, QColorFilterModel::Lightness);
  ui->rbSaturation->setProperty(FILTER_KIND_PROP, QColorFilterModel::Saturation);

  //Связываем сигналы включения радиокнопок с одним общим слотом
  connect(ui->rbHue, SIGNAL(toggled(bool)),
          this, SLOT(onRbBtnToggled(bool)));
  connect(ui->rbLightness, SIGNAL(toggled(bool)),
          this, SLOT(onRbBtnToggled(bool)));
  connect(ui->rbSaturation, SIGNAL(toggled(bool)),
          this, SLOT(onRbBtnToggled(bool)));

  //Создаем модель со списком цветов
  QColorListModel* model = new QColorListModel(this);
  //Заполнение модели случайными цветами
  for(int i=0; i<15; i++){
    model->addItem(QColor(rand()%255, rand()%255, rand()%255));
  }
  //Добавление нескольких предопределенных цветов
  model->addItem(Qt::red);
  model->addItem(Qt::white);
  model->addItem(Qt::black);

  //Создание прокси модели для фильтрации
  proxyModel = new QColorFilterModel(this);
  proxyModel->setSourceModel(model);
  //Связывание сигнала изменения положения слайдера с слотом обновления фильтрации
  connect(ui->verticalSlider, SIGNAL(sliderMoved(int)),
          proxyModel, SLOT(updateFilter(int)));
  //Указываем прокси модель для отображения в списочном представлении
  ui->listView->setModel(proxyModel);
}

MainWidget::~MainWidget()
{
  delete ui;
}

void MainWidget::onRbBtnToggled(bool checked)
{
  if (checked){
    //Обновление типа фильтрации в зависимости от выбранной радиокнопки
    int intKind = sender()->property(FILTER_KIND_PROP).toInt();
    proxyModel->setFilterKind((QColorFilterModel::FilterKind)intKind);
  }
}
