#include "qcolorfiltermodel.h"
#include <QColor>
#include <QDebug>

QColorFilterModel::QColorFilterModel(QObject *parent) :
  QSortFilterProxyModel(parent)
{
}

void QColorFilterModel::setFilterKind(QColorFilterModel::FilterKind kind)
{
  //установка текущего типа фильтрации
  filterKind = kind;
  //запуск фильтрации заново
  invalidateFilter();
}

void QColorFilterModel::updateFilter(int threshold)
{
  //установка текущего порогового значения фильтра
  this->threshold = threshold;
  //запуск фильтрации заново
  invalidateFilter();
}


bool QColorFilterModel::filterAcceptsRow(int source_row, const QModelIndex &source_parent) const
{
  //Получаем индекс в модели-источнике первого элемента фильтруемой строки
  QModelIndex index = sourceModel()->index(source_row, 0, source_parent);
  //Получаем значение, возвращаемой по роли Qt::BackgroundColorRole (цвет фона)
  QVariant value = sourceModel()->data(index, Qt::BackgroundColorRole);
  //Если значение определено моделью
  if (value.isValid()) {
    //Приводим значение к типу QColor
    QColor color = value.value<QColor>();
    //Если пороговое значение = 0, считается, что фильтр выключен
    if (threshold == 0){
      return true;
    }
    //В зависимости от типа фильтрации сравнение разных параметров цвета элемента
    //с пороговым значением
    switch (filterKind) {
    case Saturation:
      return color.saturation() > threshold;
    case Hue:
      return color.hue() > threshold;
    default:
      return color.lightness() > threshold;
    }
  }
  //Если модель не устанавливает цвет фона, используем стандартный фильтр
  return QSortFilterProxyModel::filterAcceptsRow(source_row, source_parent);
}
